;| Calendar 86z 2.2
;| By: Ahmed El-Helw / Jonah Cohen
;| Copyright (C) 1999
;| ahmed@ticalc.org / ComAsYuAre@aol.com
;| ahmed.ticalc.org / members.xoom.com/ComAsYuAre

#include "asm86.h"         
#include "ti86asm.inc"
#include "ti86abs.inc"

_clrWindow		.equ		$4A86

.org _asm_exec_ram

 nop
 jp Start
 .dw 0
 .dw description

Start:
 call _runindicoff
 call _flushallmenus

NewCal:
 ld bc,$0101
 ld (day),bc				;day and week=1
 ld a,(spos)
 ld (pos),a					;starting position
 call LeapCheck				;update length of february
 call _clrLCD
;display month/year
 ld hl,Months
 ld a,(month)

;---------= Point hl to string a =---------
; by: Joe Wingerbermuhle
; Input: a=string number (0 to 255)
;	 hl->string data
; Output: hl->string

getString:
 ld b,a
 xor a
getStringL1:
 push bc
 ld c,-1
 cpir
 pop bc
 djnz getStringL1
;hl points to month

 ld bc,57*256+20
 ld (_penCol),bc
 call _vputs
 ld c,90						;just change column
 ld hl,(year)
 call DispHL
;display S, M, T, W, T, F, S
 ld bc,$0004
 ld hl,Days
PutLoop:
 ld (_penCol),bc
 ld a,(hl)
 inc hl
 call _vputmap
 ld a,c
 add a,19
 ld c,a
 cp 137						;last visible column+19
 jr nz,PutLoop

;shade month and day names
 ld hl,$fc00			;top bar
 call invert
 ld hl,$ff90			;bottom bar
 call invert

PutCalendar:
 ld a,(pos)				;position*19 = column
 dec a
 ld h,a
 ld l,19
 call MUL_HL
 ld c,l					;we have column in c

 ld a,(week)			;week*8 = row
 add a,a
 add a,a
 add a,a
 ld b,a					;and row in b

 ld a,(day)				;we want to display (day)
 call DispA				;disp a loads bc in pencol and displays day

 ld a,(pos)
 inc a
 cp 8					;if Saturday is done, we need new week
 jr nz,same_week
 ld hl,week
 inc (hl)
 ld a,1
same_week:
 ld (pos),a				;update the position [either inc or reset]

 ld a,(month)
 call GetOffset			;get the days in a month

 ld hl,day
 inc (hl)				;advance day.
 sub (hl)					;if we reached all days, then we're done
 inc a
 jr nz,PutCalendar

DoneDisp:				;basic checking for keys and jump to routines.
 call GET_KEY
 dec a
 jr z,PastYear
 dec a
 jr z,PrevMonth
 dec a
 jr z,NextMonth
 dec a
 jr z,NextYear
 cp K_EXIT-4
 jr nz,DoneDisp

exit:
 ld hl,varname-1
 rst 20h			;copy to OP1
 rst 10h			;_findsym
 ld a,b
 ld hl,year-_asm_exec_ram+4		;offset
 add hl,de			;hl=pointer to data in original prog
 adc a,0			;in case we overlapped
 call _SET_ABS_DEST_ADDR
 xor a				;no absolute addressing now
 ld hl,year	;get data from here
 call _SET_ABS_SRC_ADDR
 ld hl,4			;number of bytes to save
 call _SET_MM_NUM_BYTES
 call _mm_ldir		;copy data
 jp _clrWindow

NextYear:
 ld a,(month)
 cp 3
 call nc,new_year
 ld a,(Feb)
 call c,new_year
 sub 27				;if 28 days, then we add 1. if 29 days, then we add 2.
 jr ChangeYear

PastYear:
 ld a,(month)
 cp 3
 call c,last_year
 ld a,(Feb)
 call nc,last_year
 sub 35				;2 day shift if a leap year, 1 day if not.  extra week for reusable comparison
 cpl					;we're going the opposite direction

ChangeYear:
 ld hl,spos			;start position of the month + 1 = start position in a year., 2 days if a leap year
 add a,(hl)					;ie if Jan 99 starts on Friday, Jan 2000 = Sat.
 cp 8					;some checks on the day [if after saturday, make sunday]
 jr c,store_spos
 sub 7				;back a week
store_spos:
 ld (spos),a
 jr newcal				;draw new calendar

NextMonth:
 ld a,(pos)
 cp 8					;again deal with if it is after saturday, reset to sunday
 jr nz,JumpCheck
 ld a,1
JumpCheck:
 ld (spos),a
 ld hl,month
 inc (hl)				;next month
 ld a,13
 cp (hl)				;if month = 13, make new year
 jr nz,newcal			;else, continue...
 ld (hl),1
 call new_year			;increase the year
newcal:
 jp NewCal

PrevMonth:					;goes back a month
 ld a,(month)
 dec a
 jr nz,MonthDone			;when you subtract a month, if not 0, then set month-1
 call last_year				;else decrement year
 ld a,12					;and set month to december
MonthDone:
 ld (month),a
 call GetOffset				;find offset of the new month
 ld b,a
 ld a,(spos)
 sub b
 dec a
add_loop:
 add a,7					;get into a normal 0-6 range
 jr nc,add_loop
 inc a					;put into 1-7 range
 jr store_spos				;finished adding

new_year:
 ld hl,(year)			;increase the year
 inc hl
 jr save_year

last_year:
 ld hl,(year)			;essentially the same as NextYear
 dec hl
save_year:
 ld (year),hl
;now update february's length with LeapCheck


LeapCheck:
 push af
 ld a,28
 ld (Feb),a				;start off with feb's length=28

 ld de,4			;check if year is divisible by 4
 call isDivisible
 jr nz,NoLeap

 ld de,100
 call isDivisible
 jr nz,LeapYear				;check if divisible by 100.. if not, then leap year.

 ld de,400					;if it is.. one final check.. is it divisible by 400?
 call isDivisible
 jr nz,NoLeap						;no leap

LeapYear:
 ld hl,Feb
 inc (hl)					;29 days
NoLeap:
 pop af
 ret

isDivisible:				;z flag if year is divisible by de
 ld hl,(year)			;get year
sub_loop:
 or a
 sbc hl,de				;c flag cleared from previous comparisons
 ret z					;divisible
 ret c					;not divisible
 jr sub_loop			;we're not done yet

GetOffset:					;finds # of days in month a
 ld e,a
 ld d,0
 ld hl,Diam
 add hl,de
 ld a,(hl)
 ret

invert:
 ld b,7*16			;7 lines, 16 bytes per line
invert_loop:
 ld a,(hl)
 cpl					;reverse the byte
 ld (hl),a
 inc hl
 djnz invert_loop
 ret


;---------= Display A or HL =------------
; by: Matthew Shepcar
DispA:
 ld l,a
 ld h,0
DispHL:
 ld (_penCol),bc
 xor a
 ld de,-1
 ld (_curRow),de
 call 4A33h
 dec hl
 jp _vputs

;-----------=  Text  =------------
Days:	    .db     "SMTWTFS"

varname:
	.db 3
description:
	.db "Cal 2.2 by Ahmed/Jonah"

Months:	    .db     0
	    .db     "January",0
	    .db     "February",0
	    .db     "March",0
	    .db     "April",0
	    .db     "May",0
	    .db     "June",0
	    .db     "July",0
	    .db     "August",0
	    .db     "September",0
	    .db     "October",0
	    .db     "November",0
	    .db     "December"


Feb		equ		$+2
Diam: .db 0,31,28,31,30,31,30,31,31,30,31,30,31			;# of days in each month

;saved vars
year:
	.dw 1999
month:
	.db 1
spos:					;starting day of month
	.db 6

;not saved
pos		equ		$
day		equ		$+1
week		equ		$+2

.end
END